@@ -1,6 +1,5 @@
 """ Unit tests for discrete variables."""
 
-import sys
 import unittest
 import copy
 
@@ -9,6 +8,7 @@ import numpy as np
 
 import openmdao.api as om
 from openmdao.core.driver import Driver
+from openmdao.devtools.debug import config_summary
 from openmdao.visualization.n2_viewer.n2_viewer import _get_viewer_data
 from openmdao.test_suite.components.sellar import StateConnection, \
      SellarDis1withDerivatives, SellarDis2withDerivatives
@@ -381,8 +381,8 @@ class DiscreteTestCase(unittest.TestCase):
             "varname  val    prom_name",
             "-------  -----  ---------",
             "_auto_ivc",
-            "  v0    [10.]  _auto_ivc.v0",
-            "  v1    11     _auto_ivc.v1",
+            "  v0    [10.]  expl.a",
+            "  v1    11     x",
             "expl",
             "  b    [20.]  expl.b",
             "  y    2      expl.y",
@@ -588,7 +588,7 @@ class DiscreteTestCase(unittest.TestCase):
         indep = model.add_subsystem('indep', om.IndepVarComp())
         indep.add_output('x', 1.0)
 
-        comp = model.add_subsystem('comp', CompDiscWDerivs())
+        model.add_subsystem('comp', CompDiscWDerivs())
         model.connect('indep.x', 'comp.x')
 
         model.add_design_var('indep.x')
@@ -608,8 +608,8 @@ class DiscreteTestCase(unittest.TestCase):
         indep = model.add_subsystem('indep', om.IndepVarComp())
         indep.add_output('x', 1.0)
 
-        comp = model.add_subsystem('comp', CompDiscWDerivsImplicit(), promotes=['N'])
-        sink = model.add_subsystem('sink', MixedCompDiscIn(1.0))
+        model.add_subsystem('comp', CompDiscWDerivsImplicit(), promotes=['N'])
+        model.add_subsystem('sink', MixedCompDiscIn(1.0))
         model.connect('indep.x', 'comp.y2_actual')
         model.connect('comp.Nout', 'sink.x')
 
@@ -635,7 +635,7 @@ class DiscreteTestCase(unittest.TestCase):
 
         G = model.add_subsystem('G', om.Group(), promotes_inputs=['x'])
 
-        G1 = G.add_subsystem('G1', InternalDiscreteGroup(), promotes_inputs=['x'], promotes_outputs=['y'])
+        G.add_subsystem('G1', InternalDiscreteGroup(), promotes_inputs=['x'], promotes_outputs=['y'])
 
         G2 = G.add_subsystem('G2', om.Group(), promotes_inputs=['x'])
         G2.add_subsystem('C2_1', om.ExecComp('y=3*x'), promotes_inputs=['x'])
@@ -659,9 +659,9 @@ class DiscreteTestCase(unittest.TestCase):
         self.assertEqual(prob['C4.y'], 16.0)
 
         with self.assertRaises(Exception) as ctx:
-            J = prob.compute_totals()
+            prob.compute_totals()
         self.assertEqual(str(ctx.exception),
-                         "Total derivative with respect to 'indep.x' depends upon discrete output variables ['G.G1.C1.y'].")
+                         "Total derivative of 'C3.y' with respect to 'x' depends upon discrete output variables ['G.G1.C1.y'].")
 
     def test_connection_to_output(self):
         prob = om.Problem(name='connection_to_output')
@@ -776,7 +776,7 @@ class DiscreteTestCase(unittest.TestCase):
 
         assert_near_equal(prob.model.get_io_metadata(includes='comp.*'), {
                           'comp.a': {'copy_shape': None,
-                                     'compute_shape': None, 
+                                     'compute_shape': None,
                                      'desc': '',
                                      'discrete': False,
                                      'distributed': False,
@@ -784,13 +784,14 @@ class DiscreteTestCase(unittest.TestCase):
                                      'global_size': 1,
                                      'has_src_indices': False,
                                      'prom_name': 'comp.a',
+                                     'require_connection': False,
                                      'shape': (1,),
                                      'shape_by_conn': False,
                                      'size': 1,
                                      'tags': set(),
                                      'units': None},
                           'comp.b': {'copy_shape': None,
-                                     'compute_shape': None, 
+                                     'compute_shape': None,
                                      'desc': '',
                                      'discrete': False,
                                      'distributed': False,
@@ -1073,6 +1074,8 @@ class DiscreteFeatureTestCase(unittest.TestCase):
 
         assert_near_equal(prob.get_val('comp.x'), 3., 1e-4)
 
+        config_summary(prob)
+
 
 if __name__ == "__main__":
     unittest.main()
