@@ -12,9 +12,6 @@ from openmdao.utils.testing_utils import use_tempdirs
 # check that pyoptsparse is installed
 OPT, OPTIMIZER = set_pyoptsparse_opt('SLSQP')
 
-if OPTIMIZER:
-    from openmdao.drivers.pyoptsparse_driver import pyOptSparseDriver
-
 
 class TrajDesignParameterOptionsDictionary(om.OptionsDictionary):
 
@@ -483,9 +480,9 @@ class TestMPIColoringBug(unittest.TestCase):
                 Color the model.
                 """
                 if coloring_mod._use_total_sparsity:
-                    if self._coloring_info['coloring'] is None and self._coloring_info['dynamic']:
+                    if self._coloring_info.do_compute_coloring() and self._coloring_info['dynamic']:
                         coloring_mod.dynamic_total_coloring(self, run_model=True,
-                                                            fname=self._get_total_coloring_fname())
+                                                            fname=self._get_total_coloring_fname(mode='output'))
                         self._setup_tot_jac_sparsity()
 
         p = om.Problem()
