@@ -2,7 +2,6 @@
 Functions for making assertions about OpenMDAO Systems.
 """
 
-from math import isnan
 from fnmatch import fnmatch
 import warnings
 import unittest
@@ -114,7 +113,7 @@ def assert_warnings(expected_warnings):
 
 
 @contextmanager
-def assert_no_warning(category, msg=None):
+def assert_no_warning(category, msg=None, contains=False):
     """
     Context manager asserting that a warning is not issued.
 
@@ -124,6 +123,8 @@ def assert_no_warning(category, msg=None):
         The class of the warning.
     msg : str or None
         The text of the warning. If None then only the warning class will be checked.
+    contains : bool
+        If True, check that the warning text contains msg, rather than checking equality.
 
     Yields
     ------
@@ -143,6 +144,9 @@ def assert_no_warning(category, msg=None):
         if issubclass(warn.category, category):
             if msg is None:
                 raise AssertionError(f"Found warning: {category} {str(warn.message)}")
+            elif contains:
+                if msg in str(warn.message):
+                    raise AssertionError(f"Found warning: {category} containing '{msg}'")
             elif str(warn.message) == msg:
                 raise AssertionError(f"Found warning: {category} {msg}")
 
@@ -415,7 +419,7 @@ def assert_no_dict_jacobians(system, include_self=True, recurse=True):
         raise AssertionError('\n'.join(parts))
 
 
-def assert_near_equal(actual, desired, tolerance=1e-15):
+def assert_near_equal(actual, desired, tolerance=1e-15, tol_type='rel'):
     """
     Check relative error.
 
@@ -431,7 +435,12 @@ def assert_near_equal(actual, desired, tolerance=1e-15):
     desired : float, array-like, dict
         The value expected.
     tolerance : float
-        Maximum relative error ``(actual - desired) / desired``.
+        Maximum relative or absolute error.
+        For relative tolerance: ``(actual - desired) / desired``.
+        For absolute  tolerance: ``(actual - desired)``.
+    tol_type : {'rel', 'abs'}
+        Type of error to use: 'rel' for relative error, 'abs' for absolute error.
+        Default is set to 'rel'.
 
     Returns
     -------
@@ -500,7 +509,7 @@ def assert_near_equal(actual, desired, tolerance=1e-15):
 
         for key in actual_keys:
             try:
-                new_error = assert_near_equal(actual[key], desired[key], tolerance)
+                new_error = assert_near_equal(actual[key], desired[key], tolerance, tol_type)
                 error = max(error, new_error)
             except ValueError as exception:
                 msg = '{}: '.format(key) + str(exception)
@@ -564,22 +573,22 @@ def assert_near_equal(actual, desired, tolerance=1e-15):
                 else:
                     raise ValueError('actual and desired values have non-matching nan'
                                      ' values')
-            if np.linalg.norm(desired) == 0:
-                error = np.linalg.norm(actual)
+            if np.linalg.norm(desired) == 0 or tol_type == 'abs':
+                error = np.linalg.norm(actual - desired)
             else:
                 error = np.linalg.norm(actual - desired) / np.linalg.norm(desired)
 
             if abs(error) > tolerance:
                 if actual.size < 10 and desired.size < 10:
-                    raise ValueError('actual %s, desired %s, rel error %s, tolerance %s'
-                                     % (actual, desired, error, tolerance))
+                    raise ValueError('actual %s, desired %s, %s error %s, tolerance %s'
+                                     % (actual, desired, tol_type, error, tolerance))
                 else:
                     raise ValueError('arrays do not match, rel error %.3e > tol (%.3e)' %
                                      (error, tolerance))
     elif isinstance(actual, tuple) and isinstance(desired, tuple):
         error = 0.0
         for act, des in zip(actual, desired):
-            new_error = assert_near_equal(act, des, tolerance)
+            new_error = assert_near_equal(act, des, tolerance, tol_type)
             error = max(error, new_error)
     else:
         raise ValueError(
